<?php
/**
 * OnPay.ru payment system callback script (API).
 *
 * <lang:ru>
 * Скрипт, который будет запрошен системой OnPay для подтверждения наличия
 * заказа и успешного прохождения платежа.
 * </lang:ru>
 *
 * @copyright atonator.com. All rights reserved.
 * @category  ATO
 * @package   Driver_PaymentSystem
 * @version   1.0
 */

require_once 'ami_env.php';

require_once dirname(__FILE__) . '/_local/eshop/AtoPaymentSystem.php';

class ATO_Onpay_Callback
{
    /**#@+
     * OnPay response code
     *
     * <lang:ru>
     * Код для ответа в формате XML системе OnPay.
     * </lang:ru>
     */

    const ERROR_PAYMENT_CANCELLED = 2;
    const ERROR_INVALID_REQUEST_PARAMS = 3;
    const ERROR_INVALID_REQUEST_SIGN = 7;
    const ERROR_TEMPORARY = 10;

    /**#@-*/

    /**
     * @var AMI_Response
     */
    private $_response;

    /**
     * @var AMI_RequestHTTP
     */
    private $_request;

    /**
     * @var string
     */
    private $_secretKey;

    /**
     * Request obligatory parameters
     *
     * @var array
     */
    private $_obligatoryParams =
        array(
            'pay_for',
            'order_amount',
            'order_currency',
            // 'balance_amount',
            // 'balance_currency',
            // 'exchange_rate',
            'type',
            'md5'
        );

    /**
     * Constructor.
     *
     * @param array  $request    GET-request parameters
     * @param string $secretKey  Secret key
     */
    public function __construct(array $request, $secretKey)
    {
        $this->_response = AMI::getSingleton('response');
        $this->_response->start();

        $this->_request =
            $request +
            array(
                'type'    => '',
                'pay_for' => ''
            );
        $this->_secretKey = (string)$secretKey;

        $this->validateRequestParams();
        $this->processRequest();
    }

    /**
     * Validates request parameters.
     *
     * <lang:ru>
     * Проверка параметров запроса.
     * </lang:ru>
     *
     * @return void
     */
    private function validateRequestParams()
    {
        switch ($this->_request['type']) {
            case 'pay':
                $this->_obligatoryParams +=
                    array(
                        'onpay_id',
                        'paymentDateTime'
                    );
                break;
            case 'check':
                break;
            default:
                $this->sendResponse(
                    self::ERROR_INVALID_REQUEST_PARAMS,
                    'Invalid request type'
                );
        }
        foreach ($this->_obligatoryParams as $param) {
            if(!isset($this->_request[$param])) {
                $this->sendResponse(
                    self::ERROR_INVALID_REQUEST_PARAMS,
                    "Missing obligatory parameter '{$param}'"
                );
            }
        }

        // type;pay_for;order_amount;order_currency;secret_key_for_api_in
        // type;pay_for;onpay_id;order_amount;order_currency;secret_key_for_api_in
        $sign =
            strtoupper(
                md5(
                    $this->_request['type'] . ';' .
                    $this->_request['pay_for'] . ';' .
                    (
                        $this->_request['type'] === 'check'
                            ? ''
                            : $this->_request['onpay_id'] . ';'
                    ) .
                    $this->_request['order_amount'] . ';' .
                    $this->_request['order_currency'] . ';' .
                    $this->_secretKey
                )
            );
        if ($this->_request['md5'] != $sign) {
            $this->sendResponse(
                self::ERROR_INVALID_REQUEST_PARAMS,
                'Invalid sign'
            );
        }
    }

    /**
     * Processes request, generates response for OnPay.
     *
     * <lang:ru>
     * Обработка запроса и отдача ответа в формате XML системе OnPay.
     * </lang:ru>
     *
     * @return void
     */
    private function processRequest()
    {
        if ($this->_request['type'] == 'check') {
            /**
             * @var AMI_ModTableItem
             */
            $oItem =
                AMI::getResourceModel('eshop_order/table')
                    ->find($this->_request['pay_for'], array('*'));
            if ($oItem->getId()) {
                if (
                    $oItem->status === 'cancelled' &&
                    $oItem->status === 'rejected'
                ) {
                    $this->sendResponse(
                        self::ERROR_PAYMENT_CANCELLED,
                        'Order is ' . $oItem->status
                    );
                } elseif (
                    ($oItem->total + $oItem->shipping)!= $this->_request['order_amount']
                ) {
                    $this->sendResponse(
                        self::ERROR_INVALID_REQUEST_PARAMS,
                        'Invalid order amount'
                    );
                }
                $this->sendResponse();
            } else {
                $this->sendResponse(
                    self::ERROR_INVALID_REQUEST_PARAMS,
                    'Invalid order id'
                );
            }
        } else {
            $httpRequest = new AMI_HTTPRequest();
            $response = $httpRequest->send(
                $GLOBALS['ROOT_PATH_WWW'] . 'eshop_final.php',
                array(
                    'order_id'     => $this->_request['pay_for'],
                    'order_amount' => $this->_request['order_amount'],
                    'sign'         => md5(
                        $this->_request['pay_for'] .
                        'ato_onpay' .
                        $this->_request['order_amount'] .
                        $this->_secretKey
                    )
                ),
                AMI_HTTPRequest::METHOD_GET
            );
            if(preg_match('/ato_onpay\[\d\]/', $response, $matches)) {
                switch ($matches[1]) {
                    case 0:
                        $this->sendResponse();
                        break;
                    case 1:
                        $this->sendResponse(
                            self::ERROR_TEMPORARY,
                            'Invalid parameters (internal)'
                        );
                        break;
                    case 2:
                        $this->sendResponse(
                            self::ERROR_INVALID_REQUEST_SIGN,
                            'Ivalid sign (internal)'
                        );
                        break;
                }
            } else {
                $this->sendResponse(
                    self::ERROR_TEMPORARY,
                    'Temporary problems (internal)'
                );
            }
        }
    }

    /**
     * Returns response sign.
     *
     * <lang:ru>
     * Получение подписи для ответа в формате XML системе OnPay.
     * </lang:ru>
     *
     * @param  int $code  Response code
     * @return string
     */
    private function getResponseSign($code)
    {
        // Defaults for missing request perameters
        $request =
            $this->_request +
            array(
                'type'           => '',
                'pay_for'        => '',
                'onpay_id'       => '',
                'order_amount'   => '',
                'order_currency' => ''
            );
        return
            strtoupper(
                md5(
                    $this->_request['type'] === 'check'
                        ?
    // type;pay_for;order_amount;order_currency;code;secret_key_api_in
                            $request['type'] . ';' .
                            $request['pay_for'] . ';' .
                            $request['order_amount'] . ';' .
                            $request['order_currency'] . ';' .
                            $code . ';' .
                            $this->_secretKey
                        :
    // type;pay_for;onpay_id;order_id;order_amount;order_currency;code;secret_key_api_in
    // order_id === pay_for
                            $request['type'] . ';' .
                            $request['pay_for'] . ';' .
                            $request['onpay_id'] . ';' .
                            $request['pay_for'] . ';' .
                            $request['order_amount'] . ';' .
                            $request['order_currency'] . ';' .
                            $code . ';' .
                            $this->_secretKey
                )
            );
    }

    /**
     * Sends XML response back to the OnPay.
     *
     * <lang:ru>
     * Выдача ответа в формате XML системе OnPay.
     * </lang:ru>
     *
     * @param  int    $code     Response code
     * @param  string $message  Response Message
     * @return void
     */
    private function sendResponse($code = 0, $message = 'OK')
    {
        $sign = $this->getResponseSign($code);
        $orderId =
            isset($this->_request['pay_for'])
                ? $this->_request['pay_for']
                : '';
        if ($this->_request['type'] == 'check') {
            $response =
    <<<EOT
<?xml version="1.0" encoding="UTF-8"?>
<result>
    <code>{$code}</code>
    <pay_for>{$orderId}</pay_for>
    <comment>{$message}</comment>
    <md5>{$sign}</md5>
</result>
EOT;
        } else {
            $onpayId = $this->_request['onpay_id'];
            $response =
<<<EOT
<?xml version="1.0" encoding="UTF-8"?>
<result>
    <code>{$code}</code>
    <comment>{$message}</comment>
    <onpay_id>{$onpayId}</onpay_id>
    <pay_for>{$orderId}</pay_for>
    <order_id>{$orderId}</order_id>
    <md5>{$sign}</md5>
</result>
EOT;
        }
        $this->_response->write($response);
        $this->_response->send();
    }
}

$secretKey = AtoPaymentSystem::getDriverParameter(
    'ato_onpay',
    'onpay_secret_key'
);
$atoOnPayCallback = new ATO_Onpay_Callback($_GET, $secretKey);
